/** Master */

#include <Wire.h>
// note: an int is two bytes: http://www.ladyada.net/learn/arduino/lesson4.html

byte GAME_IS_OURS = 0;
byte GAME_IS_THEIRS = 1;

byte move_count = 0; // Let's hope we never have a board which allows more than 255 moves
byte mode = GAME_IS_OURS; // we begin, because we are the master
byte next_move = 0; // which column we want to put a block in
byte opponent_last_move = 0;

void setup()
{
  Wire.begin(); // join i2c bus (address optional for master)
  Serial.begin(115200);               // 115200 is the default Arduino Bluetooth speed
}

void loop()
{
  Serial.println("Loop");
  mode = GAME_IS_OURS;

  next_move = calculate_next_move();
  move_count++;

  Serial.println("Sending our move");  
  // Send our move to opponent
  sendBoardStatusWire();
  mode = GAME_IS_THEIRS;
  
  // TODO check if the opponent received that message
  // TODO check if our last move was winning (and stop if it was)
  
  Serial.println("Receiving now");
  // Get the opponent's move:
  while(!receiveWireMessage())
  {  
    Wire.requestFrom(2, 3);
    delay(500);
  }
    
  // TODO check if last opponent move was a valid move
  // TODO check if last opponent move was winning (and stop if it was)
}

byte calculate_next_move()
{
  // TODO implement original FourInARowBot code
  return move_count % 5; // sends a new column everytime (makes horizontal four in a row)
}

void sendBoardStatusWire()
{
  Wire.beginTransmission(2); // transmit to device #4
  Wire.send('S');        // sends one bytes
  Wire.send(move_count);
  Wire.send(next_move);
  Wire.endTransmission();    // stop transmitting
}

boolean receiveWireMessage()
{
  // the only message we will accept is {<char>, <byte>, <byte>}, with <char> == "M"
  char control = 'X';
  byte opponent_move_count = -1;
  byte opponent_move = -1;

  if(Wire.available())
    control = Wire.receive();
  else
    return false;
  if(Wire.available())
    opponent_move_count = Wire.receive();
  else
    return false;
  if(Wire.available())
    opponent_move = Wire.receive();
  else
    return false;

  if(control != 'M')
    return false;
  if(opponent_move != (move_count + 1))
    return false;
  
  opponent_last_move = opponent_move;
  move_count++;

  return true;
}

/** Slave */
-------
#include <Wire.h>

byte GAME_IS_OURS = 0;
byte GAME_IS_THEIRS = 1;

byte move_count = 0; // Let's hope we never have a board which allows more than 255 moves
byte mode = GAME_IS_THEIRS; // the opponent starts, because we are such nice people
byte next_move = 0; // which column we want to put a block in
byte opponent_last_move = 0;

void setup()
{
  Wire.begin(2);                // join i2c bus with address #2
  Wire.onReceive(receiveEvent); // register event
  Wire.onRequest(requestEvent); // register event
  Serial.begin(9600);           // start serial for output
}

void loop()
{
  delay(100);
  
  // The game is ours, we have to do something!
  if(mode == GAME_IS_OURS)
  {
    // TODO the next line should be made smarter!
    next_move = opponent_last_move; // <- we block a vertical four in a row
    
    mode = GAME_IS_THEIRS; // we've 'calculated' our move, now it's up to them!
    move_count ++;
  }
}

// function that executes whenever data is received from master
// this function is registered as an event, see setup()
void receiveEvent(int howMany)
{
  char control = 'A';
  byte opponent_move_count = -1;
  byte opponent_move = -1;
  
  // message should equal 3 characters
  if(howMany != 3)
    return;
  
  if(Wire.available())
    control = Wire.receive(); // receive byte as a character
  if(Wire.available())
    opponent_move_count = Wire.receive();    // receive byte as an integer
  if(Wire.available())
    opponent_move = Wire.receive();    // receive byte as an integer
  
  // Let's check if the message was correctly received. We expect an "S" first:
  if(control != 'S')
    return;
  
  // then we expect that the move_count is one more than we had:
  if(opponent_move_count != move_count + 1)
    return;
  
  // We received a correct message, let's store it so we can calculate our next move
  opponent_last_move = opponent_move;
  move_count ++;
  
  // The ball is in our park now :)
  mode = GAME_IS_OURS;
}

// function that executes whenever data is requested by master
// this function is registered as an event, see setup()
void requestEvent()
{
  // The opponent asks us to send our new move
  // We'll send it only when we've calculated it
  if(mode == GAME_IS_THEIRS)
  {
    Wire.send('M');
    Wire.send(move_count);
    Wire.send(next_move);
  }
  else
  {
    // we're not ready yet!
    Wire.send('A');
    Wire.send(0);
    Wire.send(0);
  }
}
