#include <FourInARow.h>
#include <FourInARowBot.h>
//#include <iostream>
#include <Wire.h>

using namespace std;

int nColumns = 7; // <- if changed, change allMoves
int nRows = 6; // <- if changed, change allMoves
FourInARow m(nColumns, nRows);
int player = 0;
FourInARowBot a(&m, 1);
int allMoves[42]; // <- update this if nColumns and nRows are updated!! (nColumns * nRows)

byte GAME_IS_OURS = 0;
byte GAME_IS_THEIRS = 1;

byte move_count = 0; // Let's hope we never have a board which allows more than 255 moves
byte mode = GAME_IS_OURS; // we begin, because we are the master
byte next_move = 0; // which column we want to put a block in
byte opponent_last_move = 0;

void setup()
{
  Wire.begin(); // join i2c bus (address optional for master)
  Serial.begin(9600);
}

// http://www.arduino.cc/cgi-bin/yabb2/YaBB.pl?num=1269302644
boolean lineAvailable(int max_line,char *line)
{
	int c;
	static int line_idx = 0;
	boolean eol = false;
	if (max_line <= 0)    // handle bad values for max_line
	{
	  eol = true;
	  if (max_line == 0)
	    line[0] = '\0';
	}
	else		    // valid max_line
	{
	  if (Serial.available() > 0)
	  {
	    c = Serial.read();
	    if (c != -1)  // got a char -- should always be true
	    {
		if (c == '\r')
		  eol = true;
		else
		  line[line_idx++] = c;
		if (line_idx >= max_line)
		  eol = true;
		line[line_idx] = '\0';     // always terminate line, even if unfinished
		if (eol)
		  line_idx = 0;	     // reset for next line
		}
	    }
	  }
	  return eol;
} 

void stop_if_winning()
{
  Serial.println();
  // If the game is over, we stop playing:
  if(m.getState() != 0)
  {
	if(m.getState() == -1)
		Serial.println("It was a draw!");
	else
        {
		Serial.print("Player ");
                Serial.print(m.getState());
                Serial.println(" has won");
        }
        
        m.print();
        
        Serial.println("All moves:");
        for(int i = 0; i < move_count; i++)
        {
          Serial.print(allMoves[i]);
          Serial.print(", ");
        }
        // stopping program:
        while(true)
          {;}
  }
}

void loop()
{
  move_count++;

  mode = GAME_IS_OURS;

  Serial.print("Calculating move...");
  // Our turn
  next_move = a.calculateNextMove();
  m.addBlock(next_move, 1);
  Serial.println(" Done.");
  
  m.print();
  
  // add to allmoves, count starts at 1
  allMoves[move_count - 1] = next_move;
  
  Serial.print("Sending to opponent...");
  // Send our move to opponent
  sendBoardStatusWire();
  Serial.println(" Done.");
  
  stop_if_winning();
  mode = GAME_IS_THEIRS;
  
  // TODO check if the opponent received that message
  
  Serial.print("Getting opponent's move...");
  // Get the opponent's move:
  while(!receiveWireMessage())
  {  
    Wire.requestFrom(4, 3);
    delay(500);
  }
  Serial.println(" Done.");
  
  m.addBlock(opponent_last_move, 2);
  
  m.print();
  
  // add to allmoves, count starts at 1
  allMoves[move_count - 1] = opponent_last_move;
  
  stop_if_winning();
  
  delay(2000);
}

void sendBoardStatusWire()
{
  Wire.beginTransmission(4); // transmit to device #4
  Wire.write('S');        // sends one bytes
  Wire.write(move_count);
  Wire.write(next_move);
  Wire.endTransmission();    // stop transmitting
}

boolean receiveWireMessage()
{
  // the only message we will accept is {<char>, <byte>, <byte>}, with <char> == "M"
  char control = 'X';
  byte opponent_move_count = 1;
  byte opponent_move = 1;

  if(Wire.available())
  {
    control = Wire.read();
  }
  else
  {
    Serial.println("control not available");
    return false;
  }
    
  if(Wire.available())
  {
    opponent_move_count = Wire.read();
  }
  else
  {
    Serial.println("Move count not available");
    return false;
  }
  if(Wire.available())
  {
    opponent_move = Wire.read();
  }
  else
  {
    Serial.println("Opponent move not available");
    return false;
  }

  if(control != 'M')
  {
    Serial.print("Control wrong: ");
    Serial.println(control);
    
    return false;
  }
    
  if(opponent_move_count != (move_count + 1))
  {
    Serial.print("Opponent move count wrong: ");
    Serial.println(opponent_move_count);
    return false;
  }
  
  opponent_last_move = opponent_move;
  move_count++;

  return true;
}
